<?php

namespace App\Http\Controllers;

// use Illuminate\Support\Facades\Request;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Http\Request;

use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Product_feature;
use App\Mixer_config;
use App\Mixer_config_help;
use App\Mixer_channel;
use App\Mixer_channel_help;
use App\Station;
use App\Pagingzone;
use App\Zones_queue_subs;
use App\Help;
use Session;
use Gate;
use Auth;

class MixerConfigController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
         return redirect('dashboard');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
         return redirect('dashboard');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
         return redirect('dashboard');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
         return redirect('dashboard');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        // restrict by permission
		if(Gate::denies('edit_stations') || Gate::denies('view_facilities') || Gate::denies('view_stations')) {		
			// send growl notification
			session()->flash('growl_notification.message','You no longer have permissions to edit mixer configurations.');
			session()->flash('growl_notification.level','danger');
			return redirect('dashboard');
		} 
		
		// make sure id exists and is a number
        $mixer_config = Mixer_config::where('id', '=' , $id)->get()->first();
		if(count($mixer_config)==0 || !ctype_digit($id) ){
			// send growl notification
			session()->flash('growl_notification.message','That record does not exist.');
			session()->flash('growl_notification.level','danger');
			return redirect('station');
		}
		
		$product_feature = Product_feature::where('product_feature.name', 'like', 'mixer_config')->get()->first();
		$product_feature_channel = Product_feature::where('product_feature.name', 'like', 'mixer_channel')->get()->first();
		$mixer_config_help = Mixer_config_help::first();
		$mixer_channels = Mixer_channel::where('extension','like',$mixer_config->extension)->orderBy('channel', 'asc')->get();
		$mixer_channel_help = Mixer_channel_help::first();
		$station = Station::where('extension','like',$mixer_config->extension)->first();
		$station_id = $station->id;
		$station_name = $station->name;
		$station_extension = $station->extension;
		$station_ip_address = $station->ip_address;
		$amp_id = $station->amplifier_id;
		$ptt_stations = Station::where('amplifier_id','=',$amp_id)->orderBy('extension','ASC')->get();
		$station_device_type = $station->device_type;
		
		$pagingzones = Pagingzone::whereIn('type', [1,3,5,6])->get();
		
		// help topics
		$topicurl = '';
		$help = Help::where('mapid','like','mixer_config_edit')->get()->first();
		if(count($help)>0){
			$topicurl = $help->topicurl;
		}
		
		return view('mixer_config.edit', compact('mixer_config','mixer_config_help','product_feature','product_feature_channel','station_id','station_name','station_extension','station_ip_address','pagingzones','mixer_channels','mixer_channel_help','topicurl','amp_id','ptt_stations','station_device_type'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Requests\MixerConfigRequest $request, $id)
    {
        // restrict by permission
		if(Gate::denies('edit_stations') || Gate::denies('view_facilities') || Gate::denies('view_stations')) {		
			// send growl notification
			session()->flash('growl_notification.message','You no longer have permissions to edit mixer configurations.');
			session()->flash('growl_notification.level','danger');
			return redirect('dashboard');
		} 
		
		// make sure id exists and is a number
        $mixer_config = Mixer_config::where('id', '=' , $id)->get()->first();
		if(count($mixer_config)==0 || !ctype_digit($id) ){
			// send growl notification
			session()->flash('growl_notification.message','That record does not exist.');
			session()->flash('growl_notification.level','danger');
			return redirect('station');
		}
		
		$ptt_type = $request->input('ptt_type');
		$ptt_type_old = $mixer_config->ptt_type;
		$ptt_zones = $request->input('ptt_zones');
		$pagingzones = Pagingzone::whereIn('type', [1,3,5,6])->get();
		$ptt_channel = $request->input('ptt_channel');
		$ptt_channel_old = $mixer_config->ptt_channel;
		$ptt_extension = $request->input('ptt_extension');
		$extension = $mixer_config->extension;
		
		// ptt_zones required if ptt_type is Zone-Page
		if($ptt_type=='Zone-Page' && $ptt_zones==''){
			// send growl notification
			session()->flash('growl_notification.message','Push To Talk Zone is required for Push To Talk Type Zone-Page.');
			session()->flash('growl_notification.level','danger');
			return redirect()->back()->withInput();
		}
		// ptt_zones required if ptt_type is Record-Zone-Page
		if($ptt_type=='Record-Zone-Page' && $ptt_zones==''){
			// send growl notification
			session()->flash('growl_notification.message','Push To Talk Zone is required for Push To Talk Type Record-Zone-Page.');
			session()->flash('growl_notification.level','danger');
			return redirect()->back()->withInput();
		}
		
		$station = Station::where('extension','like',$mixer_config->extension)->first();
		$station_id = $station->id;
		$amplifier_id = $station->amplifier_id;
	
		// update record
		$mixer_config->update($request->all());
		
		// clear ptt_zones if ptt_type not Zone Page
		if($ptt_type!='Zone-Page' && $ptt_type!='Record-Zone-Page'){
			$mixer_config->update(['ptt_zones' => '']);
		} else if(($ptt_type=='Zone-Page' || $ptt_type=='Record-Zone-Page') && $ptt_zones=='' && count($pagingzones)>0){
			$mixer_config->update(['ptt_zones' => $pagingzones[0]->zone_number]);
		}
		
		// if ptt_type=Line-Output set mixer_channel.send_to_line_output=TRUE where mixer_channel.channel == mixer_config.ptt_channel
		if($ptt_type=='Line-Output' && $ptt_type_old!='Line-Output' && $ptt_channel>0){
			Mixer_channel::where('channel', 'like', $ptt_channel)->where('extension','=',$extension)->update([ 'send_to_line_output' => '1']);
		}
		// if ptt_channel changes disable send_to_line_output on the old channel and enable on the new channel
		if($ptt_type=='Line-Output' && $ptt_type_old=='Line-Output' && $ptt_channel>0 && $ptt_channel != $ptt_channel_old){
			Mixer_channel::where('channel', 'like', $ptt_channel_old)->where('extension','=',$extension)->update([ 'send_to_line_output' => '0']);
			Mixer_channel::where('channel', 'like', $ptt_channel)->where('extension','=',$extension)->update([ 'send_to_line_output' => '1']);
		}
		// if the ptt_type is changed from Line-Output to another type, disable send_to_line_output for the new channel
		if($ptt_type_old=='Line-Output' && $ptt_type!='Line-Output' && $ptt_channel == $ptt_channel_old){
			Mixer_channel::where('channel', 'like', $ptt_channel)->where('extension','=',$extension)->update([ 'send_to_line_output' => '0']);
		}
		// if the ptt_type = Line-Output and the ptt_channel is changed to Not Enabled, disable send_to_line_output on the old channel
		if($ptt_type_old=='Line-Output' && $ptt_type=='Line-Output' && $ptt_channel_old>0 && $ptt_channel==0){
			Mixer_channel::where('channel', 'like', $ptt_channel_old)->where('extension','=',$extension)->update([ 'send_to_line_output' => '0']);
		}
		
		if($ptt_extension!=''){
			$mixer_config->update(['extension' => $ptt_extension]);
		}
		
		// run server script
		$script_path = '/usr/local/bin/create-nyquist-cfg add '.$mixer_config->extension;
		$run_script = shell_exec($script_path);
		
		if($amplifier_id>0){
			// run server script
			$script_path = '/usr/local/bin/create-amplifier-cfg add '.$amplifier_id;
			$run_script = shell_exec($script_path);
		}
				
		// send growl notification
		session()->flash('growl_notification.message','PTT Configuration Updated.');
		session()->flash('growl_notification.level','success');
		
		if($amplifier_id>0){
			return redirect('/amplifier/');
		} else {
			return redirect('/station/'.$station_id.'/edit');
		}
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
         return redirect('dashboard');
    }
	
	public function get_ptt_zones($data){
		
		$dataArray = explode('&', $data);
		
		$ptt_type = $dataArray[0];
		$ptt_zones = $dataArray[1];
		
		$zonesArray = array();
		$pagingzones = Pagingzone::whereIn('type', [1,3,5,6])->get();
		$zones_queue_subs = Zones_queue_subs::
			leftJoin('pagingzones', 'pagingzones.zone_number', '=', 'zones_queue_subs.zone')
			->select('zones_queue_subs.zone as zone_number','pagingzones.name as name')
			->orderBy('zones_queue_subs.zone', 'asc')
            ->get();
		
		if($ptt_type=="Zone-Page"){
			
			 foreach($pagingzones as $pagingzone){
						 
				 array_push($zonesArray, $pagingzone->zone_number.';'. $pagingzone->name);
			 }			
		}
		if($ptt_type=="Record-Zone-Page"){
			
			foreach($zones_queue_subs as $zones_queue_sub){
				
				 array_push($zonesArray, $zones_queue_sub->zone_number.';'. $zones_queue_sub->name);
			 }
		}
		
		$zones_data = implode(",", $zonesArray);
		
		return $zones_data;
	}
}
